@@ -27,7 +27,7 @@ import httplib2
 import six
 
 from google.cloud.exceptions import make_exception
-from google.cloud.storage.connection import Connection
+from google.cloud.storage._http import Connection
 
 
 class MIMEApplicationHTTP(MIMEApplication):
@@ -44,8 +44,8 @@ class MIMEApplicationHTTP(MIMEApplication):
     :type headers:  dict
     :param headers: HTTP headers
 
-    :type body: str or None
-    :param body: HTTP payload
+    :type body: str
+    :param body: (Optional) HTTP payload
 
     """
     def __init__(self, method, uri, headers, body):
@@ -154,11 +154,11 @@ class Batch(Connection):
         :type data: str
         :param data: The data to send as the body of the request.
 
-        :type target_object: object or :class:`NoneType`
-        :param target_object: This allows us to enable custom behavior in our
-                              batch connection. Here we defer an HTTP request
-                              and complete initialization of the object at a
-                              later time.
+        :type target_object: object
+        :param target_object:
+            (Optional) This allows us to enable custom behavior in our batch
+            connection. Here we defer an HTTP request and complete
+            initialization of the object at a later time.
 
         :rtype: tuple of ``response`` (a dictionary of sorts)
                 and ``content`` (a string).
@@ -241,10 +241,10 @@ class Batch(Connection):
 
         url = '%s/batch' % self.API_BASE_URL
 
-        # Use the private ``_connection`` rather than the public
-        # ``.connection``, since the public connection may be this
+        # Use the private ``_base_connection`` rather than the property
+        # ``_connection``, since the property may be this
         # current batch.
-        response, content = self._client._connection._make_request(
+        response, content = self._client._base_connection._make_request(
             'POST', url, data=body, headers=headers)
         responses = list(_unpack_batch_response(response, content))
         self._finish_futures(responses)
