@@ -50,17 +50,22 @@ _API_ACCESS_ENDPOINT = 'https://storage.googleapis.com'
 class Blob(_PropertyMixin):
     """A wrapper around Cloud Storage's concept of an ``Object``.
 
-    :type name: string
+    :type name: str
     :param name: The name of the blob.  This corresponds to the
                  unique path of the object in the bucket.
 
     :type bucket: :class:`google.cloud.storage.bucket.Bucket`
     :param bucket: The bucket to which this blob belongs.
 
-    :type chunk_size: integer
+    :type chunk_size: int
     :param chunk_size: The size of a chunk of data whenever iterating (1 MB).
                        This must be a multiple of 256 KB per the API
                        specification.
+
+    :type encryption_key: bytes
+    :param encryption_key:
+        Optional 32 byte encryption key for customer-supplied encryption.
+        See https://cloud.google.com/storage/docs/encryption#customer-supplied
     """
 
     _chunk_size = None  # Default value for each instance.
@@ -68,18 +73,19 @@ class Blob(_PropertyMixin):
     _CHUNK_SIZE_MULTIPLE = 256 * 1024
     """Number (256 KB, in bytes) that must divide the chunk size."""
 
-    def __init__(self, name, bucket, chunk_size=None):
+    def __init__(self, name, bucket, chunk_size=None, encryption_key=None):
         super(Blob, self).__init__(name=name)
 
         self.chunk_size = chunk_size  # Check that setter accepts value.
         self.bucket = bucket
         self._acl = ObjectACL(self)
+        self._encryption_key = encryption_key
 
     @property
     def chunk_size(self):
         """Get the blob's default chunk size.
 
-        :rtype: integer or ``NoneType``
+        :rtype: int or ``NoneType``
         :returns: The current blob's chunk size, if it is set.
         """
         return self._chunk_size
@@ -88,8 +94,8 @@ class Blob(_PropertyMixin):
     def chunk_size(self, value):
         """Set the blob's default chunk size.
 
-        :type value: integer or ``NoneType``
-        :param value: The current blob's chunk size, if it is set.
+        :type value: int
+        :param value: (Optional) The current blob's chunk size, if it is set.
 
         :raises: :class:`ValueError` if ``value`` is not ``None`` and is not a
                  multiple of 256 KB.
@@ -103,13 +109,13 @@ class Blob(_PropertyMixin):
     def path_helper(bucket_path, blob_name):
         """Relative URL path for a blob.
 
-        :type bucket_path: string
+        :type bucket_path: str
         :param bucket_path: The URL path for a bucket.
 
-        :type blob_name: string
+        :type blob_name: str
         :param blob_name: The name of the blob.
 
-        :rtype: string
+        :rtype: str
         :returns: The relative URL path for ``blob_name``.
         """
         return bucket_path + '/o/' + quote(blob_name, safe='')
@@ -131,7 +137,7 @@ class Blob(_PropertyMixin):
     def path(self):
         """Getter property for the URL path to this Blob.
 
-        :rtype: string
+        :rtype: str
         :returns: The URL path to this Blob.
         """
         if not self.name:
@@ -229,7 +235,7 @@ class Blob(_PropertyMixin):
 
         if credentials is None:
             client = self._require_client(client)
-            credentials = client._connection.credentials
+            credentials = client._base_connection.credentials
 
         return generate_signed_url(
             credentials, resource=resource,
@@ -248,7 +254,7 @@ class Blob(_PropertyMixin):
         :param client: Optional. The client to use.  If not passed, falls back
                        to the ``client`` stored on the blob's bucket.
 
-        :rtype: boolean
+        :rtype: bool
         :returns: True if the blob exists in Cloud Storage.
         """
         client = self._require_client(client)
@@ -258,9 +264,9 @@ class Blob(_PropertyMixin):
             query_params = {'fields': 'name'}
             # We intentionally pass `_target_object=None` since fields=name
             # would limit the local properties.
-            client.connection.api_request(method='GET', path=self.path,
-                                          query_params=query_params,
-                                          _target_object=None)
+            client._connection.api_request(
+                method='GET', path=self.path,
+                query_params=query_params, _target_object=None)
             # NOTE: This will not fail immediately in a batch. However, when
             #       Batch.finish() is called, the resulting `NotFound` will be
             #       raised.
@@ -284,7 +290,7 @@ class Blob(_PropertyMixin):
         """
         return self.bucket.delete_blob(self.name, client=client)
 
-    def download_to_file(self, file_obj, encryption_key=None, client=None):
+    def download_to_file(self, file_obj, client=None):
         """Download the contents of this blob into a file-like object.
 
         .. note::
@@ -301,10 +307,10 @@ class Blob(_PropertyMixin):
             >>> client = storage.Client(project='my-project')
             >>> bucket = client.get_bucket('my-bucket')
             >>> encryption_key = 'aa426195405adee2c8081bb9e7e74b19'
-            >>> blob = Blob('secure-data', bucket)
+            >>> blob = Blob('secure-data', bucket,
+            ...             encryption_key=encryption_key)
             >>> with open('/tmp/my-secure-file', 'wb') as file_obj:
-            >>>     blob.download_to_file(file_obj,
-            ...                           encryption_key=encryption_key)
+            >>>     blob.download_to_file(file_obj)
 
         The ``encryption_key`` should be a str or bytes with a length of at
         least 32.
@@ -315,10 +321,6 @@ class Blob(_PropertyMixin):
         :type file_obj: file
         :param file_obj: A file handle to which to write the blob's data.
 
-        :type encryption_key: str or bytes
-        :param encryption_key: Optional 32 byte encryption key for
-                               customer-supplied encryption.
-
         :type client: :class:`~google.cloud.storage.client.Client` or
                       ``NoneType``
         :param client: Optional. The client to use.  If not passed, falls back
@@ -338,30 +340,24 @@ class Blob(_PropertyMixin):
         if self.chunk_size is not None:
             download.chunksize = self.chunk_size
 
-        headers = {}
-        if encryption_key:
-            _set_encryption_headers(encryption_key, headers)
+        headers = _get_encryption_headers(self._encryption_key)
 
         request = Request(download_url, 'GET', headers)
 
-        # Use the private ``_connection`` rather than the public
-        # ``.connection``, since the public connection may be a batch. A
-        # batch wraps a client's connection, but does not store the `http`
-        # object. The rest (API_BASE_URL and build_api_url) are also defined
-        # on the Batch class, but we just use the wrapped connection since
-        # it has all three (http, API_BASE_URL and build_api_url).
-        download.initialize_download(request, client._connection.http)
+        # Use ``_base_connection`` rather ``_connection`` since the current
+        # connection may be a batch. A batch wraps a client's connection,
+        # but does not store the ``http`` object. The rest (API_BASE_URL and
+        # build_api_url) are also defined on the Batch class, but we just
+        # use the wrapped connection since it has all three (http,
+        # API_BASE_URL and build_api_url).
+        download.initialize_download(request, client._base_connection.http)
 
-    def download_to_filename(self, filename, encryption_key=None, client=None):
+    def download_to_filename(self, filename, client=None):
         """Download the contents of this blob into a named file.
 
-        :type filename: string
+        :type filename: str
         :param filename: A filename to be passed to ``open``.
 
-        :type encryption_key: str or bytes
-        :param encryption_key: Optional 32 byte encryption key for
-                               customer-supplied encryption.
-
         :type client: :class:`~google.cloud.storage.client.Client` or
                       ``NoneType``
         :param client: Optional. The client to use.  If not passed, falls back
@@ -370,19 +366,14 @@ class Blob(_PropertyMixin):
         :raises: :class:`google.cloud.exceptions.NotFound`
         """
         with open(filename, 'wb') as file_obj:
-            self.download_to_file(file_obj, encryption_key=encryption_key,
-                                  client=client)
+            self.download_to_file(file_obj, client=client)
 
         mtime = time.mktime(self.updated.timetuple())
         os.utime(file_obj.name, (mtime, mtime))
 
-    def download_as_string(self, encryption_key=None, client=None):
+    def download_as_string(self, client=None):
         """Download the contents of this blob as a string.
 
-        :type encryption_key: str or bytes
-        :param encryption_key: Optional 32 byte encryption key for
-                               customer-supplied encryption.
-
         :type client: :class:`~google.cloud.storage.client.Client` or
                       ``NoneType``
         :param client: Optional. The client to use.  If not passed, falls back
@@ -393,8 +384,7 @@ class Blob(_PropertyMixin):
         :raises: :class:`google.cloud.exceptions.NotFound`
         """
         string_buffer = BytesIO()
-        self.download_to_file(string_buffer, encryption_key=encryption_key,
-                              client=client)
+        self.download_to_file(string_buffer, client=client)
         return string_buffer.getvalue()
 
     @staticmethod
@@ -409,8 +399,7 @@ class Blob(_PropertyMixin):
 
     # pylint: disable=too-many-locals
     def upload_from_file(self, file_obj, rewind=False, size=None,
-                         encryption_key=None, content_type=None, num_retries=6,
-                         client=None):
+                         content_type=None, num_retries=6, client=None):
         """Upload the contents of this blob from a file-like object.
 
         The content type of the upload will either be
@@ -437,10 +426,10 @@ class Blob(_PropertyMixin):
             >>> client = storage.Client(project='my-project')
             >>> bucket = client.get_bucket('my-bucket')
             >>> encryption_key = 'aa426195405adee2c8081bb9e7e74b19'
-            >>> blob = Blob('secure-data', bucket)
+            >>> blob = Blob('secure-data', bucket,
+            ...             encryption_key=encryption_key)
             >>> with open('my-file', 'rb') as my_file:
-            >>>     blob.upload_from_file(my_file,
-            ...                           encryption_key=encryption_key)
+            >>>     blob.upload_from_file(my_file)
 
         The ``encryption_key`` should be a str or bytes with a length of at
         least 32.
@@ -451,7 +440,7 @@ class Blob(_PropertyMixin):
         :type file_obj: file
         :param file_obj: A file handle open for reading.
 
-        :type rewind: boolean
+        :type rewind: bool
         :param rewind: If True, seek to the beginning of the file handle before
                        writing the file to Cloud Storage.
 
@@ -461,14 +450,10 @@ class Blob(_PropertyMixin):
                      :func:`os.fstat`. (If the file handle is not from the
                      filesystem this won't be possible.)
 
-        :type encryption_key: str or bytes
-        :param encryption_key: Optional 32 byte encryption key for
-                               customer-supplied encryption.
-
-        :type content_type: string or ``NoneType``
+        :type content_type: str
         :param content_type: Optional type of content being uploaded.
 
-        :type num_retries: integer
+        :type num_retries: int
         :param num_retries: Number of upload retries. Defaults to 6.
 
         :type client: :class:`~google.cloud.storage.client.Client` or
@@ -481,13 +466,13 @@ class Blob(_PropertyMixin):
                  if the upload response returns an error status.
         """
         client = self._require_client(client)
-        # Use the private ``_connection`` rather than the public
-        # ``.connection``, since the public connection may be a batch. A
-        # batch wraps a client's connection, but does not store the `http`
-        # object. The rest (API_BASE_URL and build_api_url) are also defined
-        # on the Batch class, but we just use the wrapped connection since
-        # it has all three (http, API_BASE_URL and build_api_url).
-        connection = client._connection
+        # Use ``_base_connection`` rather ``_connection`` since the current
+        # connection may be a batch. A batch wraps a client's connection,
+        # but does not store the ``http`` object. The rest (API_BASE_URL and
+        # build_api_url) are also defined on the Batch class, but we just
+        # use the wrapped connection since it has all three (http,
+        # API_BASE_URL and build_api_url).
+        connection = client._base_connection
         content_type = (content_type or self._properties.get('contentType') or
                         'application/octet-stream')
 
@@ -510,8 +495,7 @@ class Blob(_PropertyMixin):
             'User-Agent': connection.USER_AGENT,
         }
 
-        if encryption_key:
-            _set_encryption_headers(encryption_key, headers)
+        headers.update(_get_encryption_headers(self._encryption_key))
 
         upload = Upload(file_obj, content_type, total_bytes,
                         auto_transfer=False)
@@ -561,8 +545,7 @@ class Blob(_PropertyMixin):
         self._set_properties(json.loads(response_content))
     # pylint: enable=too-many-locals
 
-    def upload_from_filename(self, filename, content_type=None,
-                             encryption_key=None, client=None):
+    def upload_from_filename(self, filename, content_type=None, client=None):
         """Upload this blob's contents from the content of a named file.
 
         The content type of the upload will either be
@@ -581,16 +564,12 @@ class Blob(_PropertyMixin):
            `lifecycle <https://cloud.google.com/storage/docs/lifecycle>`_
            API documents for details.
 
-        :type filename: string
+        :type filename: str
         :param filename: The path to the file.
 
-        :type content_type: string or ``NoneType``
+        :type content_type: str
         :param content_type: Optional type of content being uploaded.
 
-        :type encryption_key: str or bytes
-        :param encryption_key: Optional 32 byte encryption key for
-                               customer-supplied encryption.
-
         :type client: :class:`~google.cloud.storage.client.Client` or
                       ``NoneType``
         :param client: Optional. The client to use.  If not passed, falls back
@@ -601,11 +580,10 @@ class Blob(_PropertyMixin):
             content_type, _ = mimetypes.guess_type(filename)
 
         with open(filename, 'rb') as file_obj:
-            self.upload_from_file(file_obj, content_type=content_type,
-                                  encryption_key=encryption_key, client=client)
+            self.upload_from_file(
+                file_obj, content_type=content_type, client=client)
 
-    def upload_from_string(self, data, content_type='text/plain',
-                           encryption_key=None, client=None):
+    def upload_from_string(self, data, content_type='text/plain', client=None):
         """Upload contents of this blob from the provided string.
 
         .. note::
@@ -623,14 +601,10 @@ class Blob(_PropertyMixin):
         :param data: The data to store in this blob.  If the value is
                      text, it will be encoded as UTF-8.
 
-        :type content_type: string
+        :type content_type: str
         :param content_type: Optional type of content being uploaded. Defaults
                              to ``'text/plain'``.
 
-        :type encryption_key: str or bytes
-        :param encryption_key: Optional 32 byte encryption key for
-                               customer-supplied encryption.
-
         :type client: :class:`~google.cloud.storage.client.Client` or
                       ``NoneType``
         :param client: Optional. The client to use.  If not passed, falls back
@@ -640,9 +614,9 @@ class Blob(_PropertyMixin):
             data = data.encode('utf-8')
         string_buffer = BytesIO()
         string_buffer.write(data)
-        self.upload_from_file(file_obj=string_buffer, rewind=True,
-                              size=len(data), content_type=content_type,
-                              encryption_key=encryption_key, client=client)
+        self.upload_from_file(
+            file_obj=string_buffer, rewind=True, size=len(data),
+            content_type=content_type, client=client)
 
     def make_public(self, client=None):
         """Make this blob public giving all users read access.
@@ -655,6 +629,78 @@ class Blob(_PropertyMixin):
         self.acl.all().grant_read()
         self.acl.save(client=client)
 
+    def compose(self, sources, client=None):
+        """Concatenate source blobs into this one.
+
+        :type sources: list of :class:`Blob`
+        :param sources: blobs whose contents will be composed into this blob.
+
+        :type client: :class:`~google.cloud.storage.client.Client` or
+                      ``NoneType``
+        :param client: Optional. The client to use.  If not passed, falls back
+                       to the ``client`` stored on the blob's bucket.
+
+        :raises: :exc:`ValueError` if this blob does not have its
+                 :attr:`content_type` set.
+        """
+        if self.content_type is None:
+            raise ValueError("Destination 'content_type' not set.")
+        client = self._require_client(client)
+        request = {
+            'sourceObjects': [{'name': source.name} for source in sources],
+            'destination': self._properties.copy(),
+        }
+        api_response = client._connection.api_request(
+            method='POST', path=self.path + '/compose', data=request,
+            _target_object=self)
+        self._set_properties(api_response)
+
+    def rewrite(self, source, token=None, client=None):
+        """Rewrite source blob into this one.
+
+        :type source: :class:`Blob`
+        :param source: blob whose contents will be rewritten into this blob.
+
+        :type token: str
+        :param token: Optional. Token returned from an earlier, not-completed
+                       call to rewrite the same source blob.  If passed,
+                       result will include updated status, total bytes written.
+
+        :type client: :class:`~google.cloud.storage.client.Client` or
+                      ``NoneType``
+        :param client: Optional. The client to use.  If not passed, falls back
+                       to the ``client`` stored on the blob's bucket.
+
+        :rtype: tuple
+        :returns: ``(token, bytes_rewritten, total_bytes)``, where ``token``
+                  is a rewrite token (``None`` if the rewrite is complete),
+                  ``bytes_rewritten`` is the number of bytes rewritten so far,
+                  and ``total_bytes`` is the total number of bytes to be
+                  rewritten.
+        """
+        client = self._require_client(client)
+        headers = _get_encryption_headers(self._encryption_key)
+        headers.update(_get_encryption_headers(
+            source._encryption_key, source=True))
+
+        if token:
+            query_params = {'rewriteToken': token}
+        else:
+            query_params = {}
+
+        api_response = client._connection.api_request(
+            method='POST', path=source.path + '/rewriteTo' + self.path,
+            query_params=query_params, data=self._properties, headers=headers,
+            _target_object=self)
+        self._set_properties(api_response['resource'])
+        rewritten = int(api_response['totalBytesRewritten'])
+        size = int(api_response['objectSize'])
+
+        if api_response['done']:
+            return None, rewritten, size
+
+        return api_response['rewriteToken'], rewritten, size
+
     cache_control = _scalar_property('cacheControl')
     """HTTP 'Cache-Control' header for this object.
 
@@ -663,7 +709,7 @@ class Blob(_PropertyMixin):
 
     If the property is not set locally, returns ``None``.
 
-    :rtype: string or ``NoneType``
+    :rtype: str or ``NoneType``
     """
 
     content_disposition = _scalar_property('contentDisposition')
@@ -674,7 +720,7 @@ class Blob(_PropertyMixin):
 
     If the property is not set locally, returns ``None``.
 
-    :rtype: string or ``NoneType``
+    :rtype: str or ``NoneType``
     """
 
     content_encoding = _scalar_property('contentEncoding')
@@ -685,7 +731,7 @@ class Blob(_PropertyMixin):
 
     If the property is not set locally, returns ``None``.
 
-    :rtype: string or ``NoneType``
+    :rtype: str or ``NoneType``
     """
 
     content_language = _scalar_property('contentLanguage')
@@ -696,7 +742,7 @@ class Blob(_PropertyMixin):
 
     If the property is not set locally, returns ``None``.
 
-    :rtype: string or ``NoneType``
+    :rtype: str or ``NoneType``
     """
 
     content_type = _scalar_property('contentType')
@@ -707,7 +753,7 @@ class Blob(_PropertyMixin):
 
     If the property is not set locally, returns ``None``.
 
-    :rtype: string or ``NoneType``
+    :rtype: str or ``NoneType``
     """
 
     crc32c = _scalar_property('crc32c')
@@ -718,7 +764,7 @@ class Blob(_PropertyMixin):
 
     If the property is not set locally, returns ``None``.
 
-    :rtype: string or ``NoneType``
+    :rtype: str or ``NoneType``
     """
 
     @property
@@ -727,7 +773,7 @@ class Blob(_PropertyMixin):
 
         See: https://cloud.google.com/storage/docs/json_api/v1/objects
 
-        :rtype: integer or ``NoneType``
+        :rtype: int or ``NoneType``
         :returns: The component count (in case of a composed object) or
                   ``None`` if the property is not set locally. This property
                   will not be set on objects not created via ``compose``.
@@ -743,7 +789,7 @@ class Blob(_PropertyMixin):
         See: http://tools.ietf.org/html/rfc2616#section-3.11 and
              https://cloud.google.com/storage/docs/json_api/v1/objects
 
-        :rtype: string or ``NoneType``
+        :rtype: str or ``NoneType``
         :returns: The blob etag or ``None`` if the property is not set locally.
         """
         return self._properties.get('etag')
@@ -754,7 +800,7 @@ class Blob(_PropertyMixin):
 
         See: https://cloud.google.com/storage/docs/json_api/v1/objects
 
-        :rtype: integer or ``NoneType``
+        :rtype: int or ``NoneType``
         :returns: The generation of the blob or ``None`` if the property
                   is not set locally.
         """
@@ -768,7 +814,7 @@ class Blob(_PropertyMixin):
 
         See: https://cloud.google.com/storage/docs/json_api/v1/objects
 
-        :rtype: string or ``NoneType``
+        :rtype: str or ``NoneType``
         :returns: The ID of the blob or ``None`` if the property is not
                   set locally.
         """
@@ -782,7 +828,7 @@ class Blob(_PropertyMixin):
 
     If the property is not set locally, returns ``None``.
 
-    :rtype: string or ``NoneType``
+    :rtype: str or ``NoneType``
     """
 
     @property
@@ -791,7 +837,7 @@ class Blob(_PropertyMixin):
 
         See: https://cloud.google.com/storage/docs/json_api/v1/objects
 
-        :rtype: string or ``NoneType``
+        :rtype: str or ``NoneType``
         :returns: The media link for the blob or ``None`` if the property is
                   not set locally.
         """
@@ -815,8 +861,8 @@ class Blob(_PropertyMixin):
 
         See: https://cloud.google.com/storage/docs/json_api/v1/objects
 
-        :type value: dict or ``NoneType``
-        :param value: The blob metadata to set.
+        :type value: dict
+        :param value: (Optional) The blob metadata to set.
         """
         self._patch_property('metadata', value)
 
@@ -826,7 +872,7 @@ class Blob(_PropertyMixin):
 
         See: https://cloud.google.com/storage/docs/json_api/v1/objects
 
-        :rtype: integer or ``NoneType``
+        :rtype: int or ``NoneType``
         :returns: The metageneration of the blob or ``None`` if the property
                   is not set locally.
         """
@@ -852,7 +898,7 @@ class Blob(_PropertyMixin):
 
         See: https://cloud.google.com/storage/docs/json_api/v1/objects
 
-        :rtype: string or ``NoneType``
+        :rtype: str or ``NoneType``
         :returns: The self link for the blob or ``None`` if the property is
                   not set locally.
         """
@@ -864,7 +910,7 @@ class Blob(_PropertyMixin):
 
         See: https://cloud.google.com/storage/docs/json_api/v1/objects
 
-        :rtype: integer or ``NoneType``
+        :rtype: int or ``NoneType``
         :returns: The size of the blob or ``None`` if the property
                   is not set locally.
         """
@@ -877,11 +923,10 @@ class Blob(_PropertyMixin):
         """Retrieve the storage class for the object.
 
         See: https://cloud.google.com/storage/docs/storage-classes
-        https://cloud.google.com/storage/docs/nearline-storage
-        https://cloud.google.com/storage/docs/durable-reduced-availability
 
-        :rtype: string or ``NoneType``
-        :returns: If set, one of "STANDARD", "NEARLINE", or
+        :rtype: str or ``NoneType``
+        :returns: If set, one of "MULTI_REGIONAL", "REGIONAL",
+                  "NEARLINE", "COLDLINE", "STANDARD", or
                   "DURABLE_REDUCED_AVAILABILITY", else ``None``.
         """
         return self._properties.get('storageClass')
@@ -938,19 +983,34 @@ class _UrlBuilder(object):
         self._relative_path = ''
 
 
-def _set_encryption_headers(key, headers):
+def _get_encryption_headers(key, source=False):
     """Builds customer encryption key headers
 
-    :type key: str or bytes
+    :type key: bytes
     :param key: 32 byte key to build request key and hash.
 
-    :type headers: dict
-    :param headers: dict of HTTP headers being sent in request.
+    :type source: bool
+    :param source: If true, return headers for the "source" blob; otherwise,
+                   return headers for the "destination" blob.
+
+    :rtype: dict
+    :returns: dict of HTTP headers being sent in request.
     """
+    if key is None:
+        return {}
+
     key = _to_bytes(key)
-    sha256_key = hashlib.sha256(key).digest()
-    key_hash = base64.b64encode(sha256_key).rstrip()
-    encoded_key = base64.b64encode(key).rstrip()
-    headers['X-Goog-Encryption-Algorithm'] = 'AES256'
-    headers['X-Goog-Encryption-Key'] = _bytes_to_unicode(encoded_key)
-    headers['X-Goog-Encryption-Key-Sha256'] = _bytes_to_unicode(key_hash)
+    key_hash = hashlib.sha256(key).digest()
+    key_hash = base64.b64encode(key_hash).rstrip()
+    key = base64.b64encode(key).rstrip()
+
+    if source:
+        prefix = 'X-Goog-Copy-Source-Encryption-'
+    else:
+        prefix = 'X-Goog-Encryption-'
+
+    return {
+        prefix + 'Algorithm': 'AES256',
+        prefix + 'Key': _bytes_to_unicode(key),
+        prefix + 'Key-Sha256': _bytes_to_unicode(key_hash),
+    }
