@@ -20,7 +20,7 @@ import six
 
 from google.cloud._helpers import _rfc3339_to_datetime
 from google.cloud.exceptions import NotFound
-from google.cloud.iterator import Iterator
+from google.cloud.iterator import HTTPIterator
 from google.cloud.storage._helpers import _PropertyMixin
 from google.cloud.storage._helpers import _scalar_property
 from google.cloud.storage.acl import BucketACL
@@ -28,45 +28,43 @@ from google.cloud.storage.acl import DefaultObjectACL
 from google.cloud.storage.blob import Blob
 
 
-class _BlobIterator(Iterator):
-    """An iterator listing blobs in a bucket
+def _blobs_page_start(iterator, page, response):
+    """Grab prefixes after a :class:`~google.cloud.iterator.Page` started.
 
-    You shouldn't have to use this directly, but instead should use the
-    :class:`google.cloud.storage.blob.Bucket.list_blobs` method.
+    :type iterator: :class:`~google.cloud.iterator.Iterator`
+    :param iterator: The iterator that is currently in use.
 
-    :type bucket: :class:`google.cloud.storage.bucket.Bucket`
-    :param bucket: The bucket from which to list blobs.
+    :type page: :class:`~google.cloud.iterator.Page`
+    :param page: The page that was just created.
 
-    :type extra_params: dict or None
-    :param extra_params: Extra query string parameters for the API call.
-
-    :type client: :class:`google.cloud.storage.client.Client`
-    :param client: Optional. The client to use for making connections.
-                   Defaults to the bucket's client.
+    :type response: dict
+    :param response: The JSON API response for a page of blobs.
     """
-    def __init__(self, bucket, extra_params=None, client=None):
-        if client is None:
-            client = bucket.client
-        self.bucket = bucket
-        self.prefixes = set()
-        self._current_prefixes = None
-        super(_BlobIterator, self).__init__(
-            client=client, path=bucket.path + '/o',
-            extra_params=extra_params)
+    page.prefixes = tuple(response.get('prefixes', ()))
+    iterator.prefixes.update(page.prefixes)
 
-    def get_items_from_response(self, response):
-        """Yield :class:`.storage.blob.Blob` items from response.
 
-        :type response: dict
-        :param response: The JSON API response for a page of blobs.
-        """
-        self._current_prefixes = tuple(response.get('prefixes', ()))
-        self.prefixes.update(self._current_prefixes)
-        for item in response.get('items', []):
-            name = item.get('name')
-            blob = Blob(name, bucket=self.bucket)
-            blob._set_properties(item)
-            yield blob
+def _item_to_blob(iterator, item):
+    """Convert a JSON blob to the native object.
+
+    .. note::
+
+        This assumes that the ``bucket`` attribute has been
+        added to the iterator after being created.
+
+    :type iterator: :class:`~google.cloud.iterator.Iterator`
+    :param iterator: The iterator that has retrieved the item.
+
+    :type item: dict
+    :param item: An item to be converted to a blob.
+
+    :rtype: :class:`.Blob`
+    :returns: The next blob in the page.
+    """
+    name = item.get('name')
+    blob = Blob(name, bucket=iterator.bucket)
+    blob._set_properties(item)
+    return blob
 
 
 class Bucket(_PropertyMixin):
@@ -76,10 +74,9 @@ class Bucket(_PropertyMixin):
     :param client: A client which holds credentials and project configuration
                    for the bucket (which requires a project).
 
-    :type name: string
+    :type name: str
     :param name: The name of the bucket.
     """
-    _iterator_class = _BlobIterator
 
     _MAX_OBJECTS_FOR_ITERATION = 256
     """Maximum number of existing objects allowed in iteration.
@@ -87,7 +84,8 @@ class Bucket(_PropertyMixin):
     This is used in Bucket.delete() and Bucket.make_public().
     """
 
-    _STORAGE_CLASSES = ('STANDARD', 'NEARLINE', 'DURABLE_REDUCED_AVAILABILITY')
+    _STORAGE_CLASSES = ('STANDARD', 'NEARLINE', 'DURABLE_REDUCED_AVAILABILITY',
+                        'MULTI_REGIONAL', 'REGIONAL', 'COLDLINE')
 
     def __init__(self, client, name=None):
         super(Bucket, self).__init__(name=name)
@@ -103,25 +101,30 @@ class Bucket(_PropertyMixin):
         """The client bound to this bucket."""
         return self._client
 
-    def blob(self, blob_name, chunk_size=None):
+    def blob(self, blob_name, chunk_size=None, encryption_key=None):
         """Factory constructor for blob object.
 
         .. note::
           This will not make an HTTP request; it simply instantiates
           a blob object owned by this bucket.
 
-        :type blob_name: string
+        :type blob_name: str
         :param blob_name: The name of the blob to be instantiated.
 
-        :type chunk_size: integer
+        :type chunk_size: int
         :param chunk_size: The size of a chunk of data whenever iterating
                            (1 MB). This must be a multiple of 256 KB per the
                            API specification.
 
+        :type encryption_key: bytes
+        :param encryption_key:
+            Optional 32 byte encryption key for customer-supplied encryption.
+
         :rtype: :class:`google.cloud.storage.blob.Blob`
         :returns: The blob object created.
         """
-        return Blob(name=blob_name, bucket=self, chunk_size=chunk_size)
+        return Blob(name=blob_name, bucket=self, chunk_size=chunk_size,
+                    encryption_key=encryption_key)
 
     def exists(self, client=None):
         """Determines whether or not this bucket exists.
@@ -131,7 +134,7 @@ class Bucket(_PropertyMixin):
         :param client: Optional. The client to use.  If not passed, falls back
                        to the ``client`` stored on the current bucket.
 
-        :rtype: boolean
+        :rtype: bool
         :returns: True if the bucket exists in Cloud Storage.
         """
         client = self._require_client(client)
@@ -141,9 +144,9 @@ class Bucket(_PropertyMixin):
             query_params = {'fields': 'name'}
             # We intentionally pass `_target_object=None` since fields=name
             # would limit the local properties.
-            client.connection.api_request(method='GET', path=self.path,
-                                          query_params=query_params,
-                                          _target_object=None)
+            client._connection.api_request(
+                method='GET', path=self.path,
+                query_params=query_params, _target_object=None)
             # NOTE: This will not fail immediately in a batch. However, when
             #       Batch.finish() is called, the resulting `NotFound` will be
             #       raised.
@@ -168,7 +171,7 @@ class Bucket(_PropertyMixin):
         query_params = {'project': client.project}
         properties = {key: self._properties[key] for key in self._changes}
         properties['name'] = self.name
-        api_response = client.connection.api_request(
+        api_response = client._connection.api_request(
             method='POST', path='/b', query_params=query_params,
             data=properties, _target_object=self)
         self._set_properties(api_response)
@@ -187,10 +190,10 @@ class Bucket(_PropertyMixin):
     def path_helper(bucket_name):
         """Relative URL path for a bucket.
 
-        :type bucket_name: string
+        :type bucket_name: str
         :param bucket_name: The bucket name in the path.
 
-        :rtype: string
+        :rtype: str
         :returns: The relative URL path for ``bucket_name``.
         """
         return '/b/' + bucket_name
@@ -216,7 +219,7 @@ class Bucket(_PropertyMixin):
           >>> print(bucket.get_blob('/does-not-exist.txt'))
           None
 
-        :type blob_name: string
+        :type blob_name: str
         :param blob_name: The name of the blob to retrieve.
 
         :type client: :class:`~google.cloud.storage.client.Client` or
@@ -230,7 +233,7 @@ class Bucket(_PropertyMixin):
         client = self._require_client(client)
         blob = Blob(bucket=self, name=blob_name)
         try:
-            response = client.connection.api_request(
+            response = client._connection.api_request(
                 method='GET', path=blob.path, _target_object=blob)
             # NOTE: We assume response.get('name') matches `blob_name`.
             blob._set_properties(response)
@@ -246,48 +249,47 @@ class Bucket(_PropertyMixin):
                    projection='noAcl', fields=None, client=None):
         """Return an iterator used to find blobs in the bucket.
 
-        :type max_results: integer or ``NoneType``
-        :param max_results: maximum number of blobs to return.
+        :type max_results: int
+        :param max_results: (Optional) Maximum number of blobs to return.
 
-        :type page_token: string
-        :param page_token: opaque marker for the next "page" of blobs. If not
-                           passed, will return the first page of blobs.
+        :type page_token: str
+        :param page_token: (Optional) Opaque marker for the next "page" of
+                           blobs. If not passed, will return the first page
+                           of blobs.
 
-        :type prefix: string or ``NoneType``
-        :param prefix: optional prefix used to filter blobs.
+        :type prefix: str
+        :param prefix: (Optional) prefix used to filter blobs.
 
-        :type delimiter: string or ``NoneType``
-        :param delimiter: optional delimter, used with ``prefix`` to
+        :type delimiter: str
+        :param delimiter: (Optional) Delimiter, used with ``prefix`` to
                           emulate hierarchy.
 
-        :type versions: boolean or ``NoneType``
-        :param versions: whether object versions should be returned as
-                         separate blobs.
+        :type versions: bool
+        :param versions: (Optional) Whether object versions should be returned
+                         as separate blobs.
 
-        :type projection: string or ``NoneType``
-        :param projection: If used, must be 'full' or 'noAcl'. Defaults to
-                           'noAcl'. Specifies the set of properties to return.
+        :type projection: str
+        :param projection: (Optional) If used, must be 'full' or 'noAcl'.
+                           Defaults to ``'noAcl'``. Specifies the set of
+                           properties to return.
 
-        :type fields: string or ``NoneType``
-        :param fields: Selector specifying which fields to include in a
-                       partial response. Must be a list of fields. For example
-                       to get a partial response with just the next page token
-                       and the language of each blob returned:
-                       'items/contentLanguage,nextPageToken'
+        :type fields: str
+        :param fields: (Optional) Selector specifying which fields to include
+                       in a partial response. Must be a list of fields. For
+                       example to get a partial response with just the next
+                       page token and the language of each blob returned:
+                       ``'items/contentLanguage,nextPageToken'``.
 
-        :type client: :class:`~google.cloud.storage.client.Client` or
-                      ``NoneType``
-        :param client: Optional. The client to use.  If not passed, falls back
+        :type client: :class:`~google.cloud.storage.client.Client`
+        :param client: (Optional) The client to use.  If not passed, falls back
                        to the ``client`` stored on the current bucket.
 
-        :rtype: :class:`_BlobIterator`.
-        :returns: An iterator of blobs.
+        :rtype: :class:`~google.cloud.iterator.Iterator`
+        :returns: Iterator of all :class:`~google.cloud.storage.blob.Blob`
+                  in this bucket matching the arguments.
         """
         extra_params = {}
 
-        if max_results is not None:
-            extra_params['maxResults'] = max_results
-
         if prefix is not None:
             extra_params['prefix'] = prefix
 
@@ -302,15 +304,15 @@ class Bucket(_PropertyMixin):
         if fields is not None:
             extra_params['fields'] = fields
 
-        result = self._iterator_class(
-            self, extra_params=extra_params, client=client)
-        # Page token must be handled specially since the base `Iterator`
-        # class has it as a reserved property.
-        if page_token is not None:
-            # pylint: disable=attribute-defined-outside-init
-            result.next_page_token = page_token
-            # pylint: enable=attribute-defined-outside-init
-        return result
+        client = self._require_client(client)
+        path = self.path + '/o'
+        iterator = HTTPIterator(
+            client=client, path=path, item_to_value=_item_to_blob,
+            page_token=page_token, max_results=max_results,
+            extra_params=extra_params, page_start=_blobs_page_start)
+        iterator.bucket = self
+        iterator.prefixes = set()
+        return iterator
 
     def delete(self, force=False, client=None):
         """Delete this bucket.
@@ -329,7 +331,7 @@ class Bucket(_PropertyMixin):
         is to prevent accidental bucket deletion and to prevent extremely long
         runtime of this method.
 
-        :type force: boolean
+        :type force: bool
         :param force: If True, empties the bucket's objects then deletes it.
 
         :type client: :class:`~google.cloud.storage.client.Client` or
@@ -361,8 +363,8 @@ class Bucket(_PropertyMixin):
         # We intentionally pass `_target_object=None` since a DELETE
         # request has no response value (whether in a standard request or
         # in a batch request).
-        client.connection.api_request(method='DELETE', path=self.path,
-                                      _target_object=None)
+        client._connection.api_request(
+            method='DELETE', path=self.path, _target_object=None)
 
     def delete_blob(self, blob_name, client=None):
         """Deletes a blob from the current bucket.
@@ -384,7 +386,7 @@ class Bucket(_PropertyMixin):
           ... except NotFound:
           ...   pass
 
-        :type blob_name: string
+        :type blob_name: str
         :param blob_name: A blob name to delete.
 
         :type client: :class:`~google.cloud.storage.client.Client` or
@@ -403,8 +405,8 @@ class Bucket(_PropertyMixin):
         # We intentionally pass `_target_object=None` since a DELETE
         # request has no response value (whether in a standard request or
         # in a batch request).
-        client.connection.api_request(method='DELETE', path=blob_path,
-                                      _target_object=None)
+        client._connection.api_request(
+            method='DELETE', path=blob_path, _target_object=None)
 
     def delete_blobs(self, blobs, on_error=None, client=None):
         """Deletes a list of blobs from the current bucket.
@@ -450,7 +452,7 @@ class Bucket(_PropertyMixin):
         :param destination_bucket: The bucket into which the blob should be
                                    copied.
 
-        :type new_name: string
+        :type new_name: str
         :param new_name: (optional) the new name for the copied file.
 
         :type client: :class:`~google.cloud.storage.client.Client` or
@@ -470,7 +472,7 @@ class Bucket(_PropertyMixin):
             new_name = blob.name
         new_blob = Blob(bucket=destination_bucket, name=new_name)
         api_path = blob.path + '/copyTo' + new_blob.path
-        copy_result = client.connection.api_request(
+        copy_result = client._connection.api_request(
             method='POST', path=api_path, _target_object=new_blob)
         if not preserve_acl:
             new_blob.acl.save(acl={}, client=client)
@@ -492,7 +494,7 @@ class Bucket(_PropertyMixin):
         :type blob: :class:`google.cloud.storage.blob.Blob`
         :param blob: The blob to be renamed.
 
-        :type new_name: string
+        :type new_name: str
         :param new_name: The new name for this blob.
 
         :type client: :class:`~google.cloud.storage.client.Client` or
@@ -509,11 +511,14 @@ class Bucket(_PropertyMixin):
 
     @property
     def cors(self):
-        """Retrieve CORS policies configured for this bucket.
+        """Retrieve or set CORS policies configured for this bucket.
 
         See: http://www.w3.org/TR/cors/ and
              https://cloud.google.com/storage/docs/json_api/v1/buckets
 
+        :setter: Set CORS policies for this bucket.
+        :getter: Gets the CORS policies for this bucket.
+
         :rtype: list of dictionaries
         :returns: A sequence of mappings describing each CORS policy.
         """
@@ -539,7 +544,7 @@ class Bucket(_PropertyMixin):
         See: http://tools.ietf.org/html/rfc2616#section-3.11 and
              https://cloud.google.com/storage/docs/json_api/v1/buckets
 
-        :rtype: string or ``NoneType``
+        :rtype: str or ``NoneType``
         :returns: The bucket etag or ``None`` if the property is not
                   set locally.
         """
@@ -551,7 +556,7 @@ class Bucket(_PropertyMixin):
 
         See: https://cloud.google.com/storage/docs/json_api/v1/buckets
 
-        :rtype: string or ``NoneType``
+        :rtype: str or ``NoneType``
         :returns: The ID of the bucket or ``None`` if the property is not
                   set locally.
         """
@@ -582,7 +587,7 @@ class Bucket(_PropertyMixin):
 
     If the property is not set locally, returns ``None``.
 
-    :rtype: string or ``NoneType``
+    :rtype: str or ``NoneType``
     """
 
     def get_logging(self):
@@ -600,12 +605,12 @@ class Bucket(_PropertyMixin):
     def enable_logging(self, bucket_name, object_prefix=''):
         """Enable access logging for this bucket.
 
-        See: https://cloud.google.com/storage/docs/accesslogs#delivery
+        See: https://cloud.google.com/storage/docs/accesslogs
 
-        :type bucket_name: string
+        :type bucket_name: str
         :param bucket_name: name of bucket in which to store access logs
 
-        :type object_prefix: string
+        :type object_prefix: str
         :param object_prefix: prefix for access log filenames
         """
         info = {'logBucket': bucket_name, 'logObjectPrefix': object_prefix}
@@ -624,7 +629,7 @@ class Bucket(_PropertyMixin):
 
         See: https://cloud.google.com/storage/docs/json_api/v1/buckets
 
-        :rtype: integer or ``NoneType``
+        :rtype: int or ``NoneType``
         :returns: The metageneration of the bucket or ``None`` if the property
                   is not set locally.
         """
@@ -650,7 +655,7 @@ class Bucket(_PropertyMixin):
 
         See: https://cloud.google.com/storage/docs/json_api/v1/buckets
 
-        :rtype: integer or ``NoneType``
+        :rtype: int or ``NoneType``
         :returns: The project number that owns the bucket or ``None`` if the
                   property is not set locally.
         """
@@ -664,7 +669,7 @@ class Bucket(_PropertyMixin):
 
         See: https://cloud.google.com/storage/docs/json_api/v1/buckets
 
-        :rtype: string or ``NoneType``
+        :rtype: str or ``NoneType``
         :returns: The self link for the bucket or ``None`` if the property is
                   not set locally.
         """
@@ -675,11 +680,10 @@ class Bucket(_PropertyMixin):
         """Retrieve the storage class for the bucket.
 
         See: https://cloud.google.com/storage/docs/storage-classes
-        https://cloud.google.com/storage/docs/nearline-storage
-        https://cloud.google.com/storage/docs/durable-reduced-availability
 
-        :rtype: string or ``NoneType``
-        :returns: If set, one of "STANDARD", "NEARLINE", or
+        :rtype: str or ``NoneType``
+        :returns: If set, one of "MULTI_REGIONAL", "REGIONAL",
+                  "NEARLINE", "COLDLINE", "STANDARD", or
                   "DURABLE_REDUCED_AVAILABILITY", else ``None``.
         """
         return self._properties.get('storageClass')
@@ -689,12 +693,10 @@ class Bucket(_PropertyMixin):
         """Set the storage class for the bucket.
 
         See: https://cloud.google.com/storage/docs/storage-classes
-        https://cloud.google.com/storage/docs/nearline-storage
-        https://cloud.google.com/storage/docs/durable-reduced-availability
 
-        :type value: string
-        :param value: one of "STANDARD", "NEARLINE", or
-                      "DURABLE_REDUCED_AVAILABILITY"
+        :type value: str
+        :param value: one of "MULTI_REGIONAL", "REGIONAL", "NEARLINE",
+                      "COLDLINE", "STANDARD", or "DURABLE_REDUCED_AVAILABILITY"
         """
         if value not in self._STORAGE_CLASSES:
             raise ValueError('Invalid storage class: %s' % (value,))
@@ -721,7 +723,7 @@ class Bucket(_PropertyMixin):
         See:  https://cloud.google.com/storage/docs/object-versioning for
         details.
 
-        :rtype: boolean
+        :rtype: bool
         :returns: True if enabled, else False.
         """
         versioning = self._properties.get('versioning', {})
@@ -765,12 +767,12 @@ class Bucket(_PropertyMixin):
         the bucket, and anything else I add to the bucket.  Just make it
         all public."
 
-        :type main_page_suffix: string
+        :type main_page_suffix: str
         :param main_page_suffix: The page to use as the main page
                                  of a directory.
                                  Typically something like index.html.
 
-        :type not_found_page: string
+        :type not_found_page: str
         :param not_found_page: The file to use when a page isn't found.
         """
         data = {
@@ -794,11 +796,11 @@ class Bucket(_PropertyMixin):
         objects / blobs this will cowardly refuse to make the objects public.
         This is to prevent extremely long runtime of this method.
 
-        :type recursive: boolean
+        :type recursive: bool
         :param recursive: If True, this will make all blobs inside the bucket
                           public as well.
 
-        :type future: boolean
+        :type future: bool
         :param future: If True, this will make all objects created in the
                        future public as well.
 
